@@ -18,11 +18,11 @@
 using System;
 using System.Collections.Generic;
 using System.IO;
-using System.Security.Cryptography;
 using System.Text;
 using System.Text.RegularExpressions;
 using System.Threading;
-using System.Timers;
+
+using Timers = System.Timers;
 
 namespace SparkleLib {
 
@@ -36,23 +36,16 @@ namespace SparkleLib {
 
     public abstract class SparkleRepoBase {
 
-        private TimeSpan short_interval = new TimeSpan (0, 0, 3, 0);
-        private TimeSpan long_interval  = new TimeSpan (0, 0, 10, 0);
-        private TimeSpan poll_interval;
-        private SparkleWatcher watcher;
-        private SparkleListenerBase listener;
-        private System.Timers.Timer local_timer  = new System.Timers.Timer () { Interval = 0.25 * 1000 };
-        private System.Timers.Timer remote_timer = new System.Timers.Timer () { Interval = 10 * 1000 };
-        private DateTime last_poll               = DateTime.Now;
-        private List<double> size_buffer         = new List<double> ();
-        private Object change_lock               = new Object ();
-        private Object watch_lock                = new Object ();
-        private double progress_percentage       = 0.0;
-        private string progress_speed            = "";
-        private bool has_changed                 = false;
-        private bool is_buffering                = false;
-        private bool server_online               = true;
-        private SyncStatus status;
+        public abstract string CurrentRevision { get; }
+        public abstract double Size { get; }
+        public abstract double HistorySize { get; }
+        public abstract List<string> ExcludePaths { get; }
+        public abstract bool HasUnsyncedChanges { get; set; }
+        public abstract bool HasLocalChanges { get; }
+        public abstract bool HasRemoteChanges { get; }
+        public abstract bool SyncUp ();
+        public abstract bool SyncDown ();
+        public abstract List<SparkleChangeSet> GetChangeSets (int count);
 
 
         public delegate void SyncStatusChangedEventHandler (SyncStatus new_status);
@@ -70,45 +63,41 @@ namespace SparkleLib {
         public delegate void ChangesDetectedEventHandler ();
         public event ChangesDetectedEventHandler ChangesDetected;
 
-
         public readonly string LocalPath;
         public readonly string Name;
-        public readonly Uri Url;
+        public readonly Uri RemoteUrl;
+        public List<SparkleChangeSet> ChangeSets { get; protected set; }
+        public SyncStatus Status { get; private set; }
+        public bool ServerOnline { get; private set; }
+        public bool IsBuffering { get; private set; }
+        public double ProgressPercentage { get; private set; }
+        public string ProgressSpeed { get; private set; }
 
-        public abstract string Identifier { get; }
-        public abstract string CurrentRevision { get; }
-        public abstract double Size { get; }
-        public abstract double HistorySize { get; }
-        public abstract List<string> ExcludePaths { get; }
-        public abstract bool HasUnsyncedChanges { get; set; }
-        public abstract bool HasLocalChanges { get; }
-        public abstract bool HasRemoteChanges { get; }
-        public abstract bool SyncUp ();
-        public abstract bool SyncDown ();
-        public abstract List<SparkleChangeSet> GetChangeSets (int count);
-
-
-        public bool ServerOnline {
+        public string Identifier {
             get {
-                return this.server_online;
-            }
-        }
+                if (this.identifier != null)
+                    return this.identifier;
 
-        public SyncStatus Status {
-            get {
-                return this.status;
-            }
-        }
+                string id_path = Path.Combine (LocalPath, ".sparkleshare");
 
-        public double ProgressPercentage {
-            get {
-                return this.progress_percentage;
-            }
-        }
+                if (File.Exists (id_path))
+                    this.identifier = File.ReadAllText (id_path).Trim ();
 
-        public string ProgressSpeed {
-            get {
-                return this.progress_speed;
+                if (this.identifier != null && this.identifier.Length > 0) {
+                    return this.identifier;
+
+                } else {
+                    Random random   = new Random ();
+                    string number   = "" + random.Next () + "" + random.Next () + "" + random.Next ();
+                    this.identifier = SparkleHelpers.SHA1 (number);
+
+                    File.WriteAllText (id_path, this.identifier);
+                    File.SetAttributes (id_path, FileAttributes.Hidden);
+
+                    SparkleHelpers.DebugInfo ("Local", Name + " | Assigned identifier: " + this.identifier);
+
+                    return this.identifier;
+                }
             }
         }
 
@@ -118,48 +107,64 @@ namespace SparkleLib {
             }
         }
 
-        public bool IsSyncing {
+
+        private string identifier;
+
+        private SparkleWatcher watcher;
+        private SparkleListenerBase listener;
+
+        private TimeSpan poll_interval = PollInterval.Short;
+
+        private Object change_lock = new Object ();
+        private DateTime last_poll = DateTime.Now;
+
+        private Timers.Timer remote_timer = new Timers.Timer () {
+            Interval = 5000
+        };
+
+        private bool is_syncing {
             get {
-                return (Status == SyncStatus.SyncUp   ||
-                        Status == SyncStatus.SyncDown ||
-                        this.is_buffering);
+                return (Status == SyncStatus.SyncUp || Status == SyncStatus.SyncDown || IsBuffering);
             }
         }
 
-        public bool IsBuffering {
-            get {
-                return this.is_buffering;
-            }
+        private static class PollInterval {
+            public static TimeSpan Short { get { return new TimeSpan (0, 0, 5, 0); }}
+            public static TimeSpan Long  { get { return new TimeSpan (0, 0, 15, 0); }}
         }
 
 
         public SparkleRepoBase (string path)
         {
-            LocalPath = path;
-            Name      = Path.GetFileName (LocalPath);
-            Url       = new Uri (SparkleConfig.DefaultConfig.GetUrlForFolder (Name));
-
-            this.poll_interval = this.short_interval;
+            LocalPath    = path;
+            Name         = Path.GetFileName (LocalPath);
+            RemoteUrl    = new Uri (SparkleConfig.DefaultConfig.GetUrlForFolder (Name));
+            IsBuffering  = false;
+            ServerOnline = true;
 
             SyncStatusChanged += delegate (SyncStatus status) {
-                this.status = status;
+                Status = status;
             };
 
+            this.identifier = Identifier;
+
             if (CurrentRevision == null)
                 CreateInitialChangeSet ();
 
-            CreateWatcher ();
-            CreateListener ();
+            ChangeSets = GetChangeSets ();
+            this.watcher = CreateWatcher ();
 
-            this.local_timer.Elapsed += delegate (object o, ElapsedEventArgs args) {
-                CheckForChanges ();
-            };
+            new Thread (
+                new ThreadStart (delegate {
+                    CreateListener ();
+                })
+            ).Start ();
 
             this.remote_timer.Elapsed += delegate {
                 bool time_to_poll = (DateTime.Compare (this.last_poll,
                     DateTime.Now.Subtract (this.poll_interval)) < 0);
 
-                if (time_to_poll) {
+                if (time_to_poll && !is_syncing) {
                     this.last_poll = DateTime.Now;
 
                     if (HasRemoteChanges)
@@ -168,11 +173,9 @@ namespace SparkleLib {
 
                 // In the unlikely case that we haven't synced up our
                 // changes or the server was down, sync up again
-                if (HasUnsyncedChanges && !IsSyncing && this.server_online)
+                if (HasUnsyncedChanges && !is_syncing && ServerOnline)
                     SyncUpBase ();
             };
-
-
         }
 
 
@@ -181,94 +184,120 @@ namespace SparkleLib {
             // Sync up everything that changed
             // since we've been offline
             if (HasLocalChanges) {
-                DisableWatching ();
                 SyncUpBase ();
 
                 while (HasUnsyncedChanges)
                     SyncUpBase ();
-
-                EnableWatching ();
             }
 
             this.remote_timer.Start ();
-            this.local_timer.Start ();
+        }
+
+
+        // Create an initial change set when the
+        // user has fetched an empty remote folder
+        public virtual void CreateInitialChangeSet ()
+        {
+            string file_path = Path.Combine (LocalPath, "SparkleShare.txt");
+            string n         = Environment.NewLine;
+
+            File.WriteAllText (file_path,
+                "Congratulations, you've successfully created a SparkleShare repository!" + n +
+                "" + n +
+                "Any files you add or change in this folder will be automatically synced to " + n +
+                RemoteUrl + " and everyone connected to it." + n +
+                "" + n +
+                "SparkleShare is an Open Source software program that helps people " + n +
+                "collaborate and share files. If you like what we do, please consider a small " + n +
+                "donation to support the project: http://sparkleshare.org/support-us/" + n +
+                "" + n +
+                "Have fun! :)" + n
+            );
+        }
+
+
+        public List<SparkleChangeSet> GetChangeSets () {
+            return GetChangeSets (30);
+        }
+
+
+        public void OnFileActivity (FileSystemEventArgs args)
+        {
+            // Check the watcher for the occasions where this
+            // method is called directly
+            if (!this.watcher.EnableRaisingEvents || IsBuffering)
+                return;
+
+            lock (this.change_lock) {
+                string relative_path = args.FullPath.Replace (LocalPath, "");
+
+                foreach (string exclude_path in ExcludePaths) {
+                    if (relative_path.Contains (exclude_path))
+                        return;
+                }
+
+                if (!IsBuffering && HasLocalChanges) {
+                    IsBuffering = true;
+                    this.watcher.Disable ();
+                    this.remote_timer.Stop ();
+
+                    SparkleHelpers.DebugInfo ("Local", Name + " | Activity detected, waiting for it to settle...");
+
+                    if (ChangesDetected != null)
+                        ChangesDetected ();
+
+                    List<double> size_buffer = new List<double> ();
+
+                    do {
+                        if (size_buffer.Count >= 4)
+                            size_buffer.RemoveAt (0);
+
+                        DirectoryInfo info = new DirectoryInfo (LocalPath);
+                        size_buffer.Add (CalculateSize (info));
+
+                        if (size_buffer.Count >= 4 &&
+                            size_buffer [0].Equals (size_buffer [1]) &&
+                            size_buffer [1].Equals (size_buffer [2]) &&
+                            size_buffer [2].Equals (size_buffer [3])) {
+
+                            SparkleHelpers.DebugInfo ("Local", Name + " | Activity has settled");
+                            IsBuffering = false;
+
+                            this.watcher.Disable ();
+                            while (HasLocalChanges)
+                                SyncUpBase ();
+                            this.watcher.Enable ();
+
+                        } else {
+                            Thread.Sleep (500);
+                        }
+
+                    } while (IsBuffering);
+                }
+            }
         }
 
 
         protected void OnConflictResolved ()
         {
-            HasUnsyncedChanges = true; // ?
-
             if (ConflictResolved != null)
                 ConflictResolved ();
         }
 
 
-        // Disposes all resourses of this object
-        public void Dispose ()
-        {
-            this.remote_timer.Dispose ();
-            this.local_timer.Dispose ();
-            this.listener.Dispose ();
-        }
-
-
-        // Starts a timer when something changes
-        public void OnFileActivity (FileSystemEventArgs args)
-        {
-            // Check the watcher for the occasions where this
-            // method is called directly
-            if (!this.watcher.EnableRaisingEvents)
-                return;
-
-            string relative_path = args.FullPath.Replace (LocalPath, "");
-
-            foreach (string exclude_path in ExcludePaths) {
-                if (relative_path.Contains (exclude_path))
-                    return;
-            }
-
-            WatcherChangeTypes wct = args.ChangeType;
-
-            if (HasLocalChanges) {
-                this.is_buffering = true;
-
-                // We want to disable wathcing temporarily, but
-                // not stop the local timer
-                this.watcher.EnableRaisingEvents = false;
-
-                // Only fire the event if the timer has been stopped.
-                // This prevents multiple events from being raised whilst "buffering".
-                if (!this.has_changed) {
-                    if (ChangesDetected != null)
-                        ChangesDetected ();
-                }
-
-                SparkleHelpers.DebugInfo ("Event", "[" + Name + "] " + wct.ToString () + " '" + args.Name + "'");
-                SparkleHelpers.DebugInfo ("Event", "[" + Name + "] Changes found, checking if settled.");
-
-                this.remote_timer.Stop ();
-
-                lock (this.change_lock) {
-                    this.has_changed = true;
-                }
-            }
-        }
-
-
         private void SyncUpBase ()
         {
             try {
-                DisableWatching ();
+                this.watcher.Disable ();
                 this.remote_timer.Stop ();
 
-                SparkleHelpers.DebugInfo ("SyncUp", "[" + Name + "] Initiated");
+                SparkleHelpers.DebugInfo ("SyncUp", Name + " | Initiated");
 
                 if (SyncStatusChanged != null)
                     SyncStatusChanged (SyncStatus.SyncUp);
 
                 if (SyncUp ()) {
-                    SparkleHelpers.DebugInfo ("SyncUp", "[" + Name + "] Done");
+                    SparkleHelpers.DebugInfo ("SyncUp", Name + " | Done");
 
                     HasUnsyncedChanges = false;
 
@@ -278,13 +307,13 @@ namespace SparkleLib {
                     this.listener.Announce (new SparkleAnnouncement (Identifier, CurrentRevision));
 
                 } else {
-                    SparkleHelpers.DebugInfo ("SyncUp", "[" + Name + "] Error");
+                    SparkleHelpers.DebugInfo ("SyncUp", Name + " | Error");
 
                     HasUnsyncedChanges = true;
                     SyncDownBase ();
-                    DisableWatching ();
+                    this.watcher.Disable ();
 
-                    if (this.server_online && SyncUp ()) {
+                    if (ServerOnline && SyncUp ()) {
                         HasUnsyncedChanges = false;
 
                         if (SyncStatusChanged != null)
@@ -293,7 +322,7 @@ namespace SparkleLib {
                         this.listener.Announce (new SparkleAnnouncement (Identifier, CurrentRevision));
 
                     } else {
-                        this.server_online = false;
+                        ServerOnline = false;
 
                         if (SyncStatusChanged != null)
                             SyncStatusChanged (SyncStatus.Error);
@@ -302,19 +331,19 @@ namespace SparkleLib {
 
             } finally {
                 this.remote_timer.Start ();
-                EnableWatching ();
+                this.watcher.Enable ();
 
-                this.progress_percentage = 0.0;
-                this.progress_speed      = "";
+                ProgressPercentage = 0.0;
+                ProgressSpeed      = "";
             }
         }
 
 
         private void SyncDownBase ()
         {
-            SparkleHelpers.DebugInfo ("SyncDown", "[" + Name + "] Initiated");
+            SparkleHelpers.DebugInfo ("SyncDown", Name + " | Initiated");
             this.remote_timer.Stop ();
-            DisableWatching ();
+            this.watcher.Disable ();
 
             if (SyncStatusChanged != null)
                 SyncStatusChanged (SyncStatus.SyncDown);
@@ -322,15 +351,20 @@ namespace SparkleLib {
             string pre_sync_revision = CurrentRevision;
 
             if (SyncDown ()) {
-                SparkleHelpers.DebugInfo ("SyncDown", "[" + Name + "] Done");
-                this.server_online = true;
+                SparkleHelpers.DebugInfo ("SyncDown", Name + " | Done");
+                ServerOnline = true;
 
                 if (!pre_sync_revision.Equals (CurrentRevision)) {
-                    List<SparkleChangeSet> change_sets = GetChangeSets (1);
+                   if (ChangeSets != null &&
+                       ChangeSets.Count > 0) {
 
-                   if (change_sets != null && change_sets.Count > 0) {
-                        if (NewChangeSet != null)
-                            NewChangeSet (change_sets [0]);
+                        bool emit_change_event = true;
+                        foreach (SparkleChange change in ChangeSets [0].Changes)
+                            if (change.Path.EndsWith (".sparkleshare"))
+                                emit_change_event = false;
+                        
+                        if (NewChangeSet != null && emit_change_event)
+                            NewChangeSet (ChangeSets [0]);
                     }
                 }
 
@@ -338,41 +372,44 @@ namespace SparkleLib {
                 // conflict. Tries only once, then lets
                 // the timer try again periodically
                 if (HasUnsyncedChanges) {
-	                if (SyncStatusChanged != null)
-	                    SyncStatusChanged (SyncStatus.SyncUp);
-					
+                    if (SyncStatusChanged != null)
+                        SyncStatusChanged (SyncStatus.SyncUp);
+                    
                     SyncUp ();
-					HasUnsyncedChanges = false;
-				}
-				
+                    HasUnsyncedChanges = false;
+                }
+                
                 if (SyncStatusChanged != null)
                     SyncStatusChanged (SyncStatus.Idle);
 
             } else {
-                SparkleHelpers.DebugInfo ("SyncDown", "[" + Name + "] Error");
-                this.server_online = false;
+                SparkleHelpers.DebugInfo ("SyncDown", Name + " | Error");
+                ServerOnline = false;
 
                 if (SyncStatusChanged != null)
                     SyncStatusChanged (SyncStatus.Error);
             }
 
+            ProgressPercentage = 0.0;
+            ProgressSpeed      = "";
+
             if (SyncStatusChanged != null)
                 SyncStatusChanged (SyncStatus.Idle);
 
             this.remote_timer.Start ();
-            EnableWatching ();
-
-            this.progress_percentage = 0.0;
-            this.progress_speed      = "";    
-		}
+            this.watcher.Enable ();
+        }
 
 
-        private void CreateWatcher ()
+        private SparkleWatcher CreateWatcher ()
         {
-            this.watcher = new SparkleWatcher (LocalPath);
-            this.watcher.ChangeEvent += delegate (FileSystemEventArgs args) {
+            SparkleWatcher watcher = new SparkleWatcher (LocalPath);
+
+            watcher.ChangeEvent += delegate (FileSystemEventArgs args) {
                 OnFileActivity (args);
             };
+
+            return watcher;
         }
 
 
@@ -381,21 +418,22 @@ namespace SparkleLib {
             this.listener = SparkleListenerFactory.CreateListener (Name, Identifier);
 
             if (this.listener.IsConnected) {
-                this.poll_interval = this.long_interval;
+                this.poll_interval = PollInterval.Long;
 
-                new Thread (new ThreadStart (delegate {
-                    if (!IsSyncing && HasRemoteChanges)
-                        SyncDownBase ();
-                })).Start ();
+                new Thread (
+                    new ThreadStart (delegate {
+                        if (!is_syncing && HasRemoteChanges)
+                            SyncDownBase ();
+                    })
+                ).Start ();
             }
 
             // Stop polling when the connection to the irc channel is succesful
             this.listener.Connected += delegate {
-                this.poll_interval = this.long_interval;
+                this.poll_interval = PollInterval.Long;
                 this.last_poll = DateTime.Now;
 
-                if (!IsSyncing) {
-
+                if (!is_syncing) {
                     // Check for changes manually one more time
                     if (HasRemoteChanges)
                         SyncDownBase ();
@@ -406,9 +444,9 @@ namespace SparkleLib {
                 }
             };
 
-            // Start polling when the connection to the irc channel is lost
+            // Start polling when the connection to the channel is lost
             this.listener.Disconnected += delegate {
-                this.poll_interval = this.short_interval;
+                this.poll_interval = PollInterval.Short;
                 SparkleHelpers.DebugInfo (Name, "Falling back to polling");
             };
 
@@ -419,7 +457,7 @@ namespace SparkleLib {
                 if (announcement.FolderIdentifier.Equals (identifier) &&
                     !announcement.Message.Equals (CurrentRevision)) {
 
-                    while (this.IsSyncing)
+                    while (this.is_syncing)
                         System.Threading.Thread.Sleep (100);
 
                     SparkleHelpers.DebugInfo ("Listener", "Syncing due to announcement");
@@ -437,111 +475,29 @@ namespace SparkleLib {
         }
 
 
-        private void CheckForChanges ()
-        {
-            lock (this.change_lock) {
-                if (this.has_changed) {
-                    if (this.size_buffer.Count >= 4)
-                        this.size_buffer.RemoveAt (0);
-
-                    DirectoryInfo dir_info = new DirectoryInfo (LocalPath);
-                     this.size_buffer.Add (CalculateSize (dir_info));
-
-                    if (this.size_buffer.Count >= 4 &&
-                        this.size_buffer [0].Equals (this.size_buffer [1]) &&
-                        this.size_buffer [1].Equals (this.size_buffer [2]) &&
-                        this.size_buffer [2].Equals (this.size_buffer [3])) {
-
-                        SparkleHelpers.DebugInfo ("Local", "[" + Name + "] Changes have settled.");
-                        this.is_buffering = false;
-                        this.has_changed  = false;
-
-                        DisableWatching ();
-                        while (HasLocalChanges)
-                            SyncUpBase ();
-                        EnableWatching ();
-                    }
-                }
-            }
-        }
-
-
-        protected void DisableWatching ()
-        {
-            lock (this.watch_lock) {
-                this.watcher.EnableRaisingEvents = false;
-                this.local_timer.Stop ();
-            }
-        }
-
-
-        protected void EnableWatching ()
-        {
-            lock (this.watch_lock) {
-                this.watcher.EnableRaisingEvents = true;
-                this.local_timer.Start ();
-            }
-        }
-
-
         private DateTime progress_last_change     = DateTime.Now;
         private TimeSpan progress_change_interval = new TimeSpan (0, 0, 0, 1);
 
         protected void OnProgressChanged (double progress_percentage, string progress_speed)
         {
-            if (DateTime.Compare (this.progress_last_change,
-                    DateTime.Now.Subtract (this.progress_change_interval)) < 0) {
+            // Only trigger the ProgressChanged event once per second
+            if (DateTime.Compare (this.progress_last_change, DateTime.Now.Subtract (this.progress_change_interval)) >= 0)
+                return;
 
-                if (ProgressChanged != null) {
-                    if (progress_percentage == 100.0)
-                        progress_percentage = 99.0;
+            if (ProgressChanged != null) {
+                if (progress_percentage == 100.0)
+                    progress_percentage = 99.0;
 
-                    this.progress_percentage  = progress_percentage;
-                    this.progress_speed       = progress_speed;
-                    this.progress_last_change = DateTime.Now;
+                ProgressPercentage = progress_percentage;
+                ProgressSpeed      = progress_speed;
 
-                    ProgressChanged (progress_percentage, progress_speed);
-                }
+                this.progress_last_change = DateTime.Now;
+
+                ProgressChanged (progress_percentage, progress_speed);
             }
         }
 
 
-        // Create an initial change set when the
-        // user has fetched an empty remote folder
-        private void CreateInitialChangeSet ()
-        {
-            string file_path = Path.Combine (LocalPath, "SparkleShare.txt");
-            TextWriter writer = new StreamWriter (file_path);
-
-            writer.WriteLine ("Congratulations, you've successfully created a SparkleShare repository!");
-            writer.WriteLine ("");
-            writer.WriteLine ("Any files you add or change in this folder will be automatically synced to ");
-            writer.WriteLine (Url + " and everyone connected to it.");
-
-            writer.WriteLine ("");
-            writer.WriteLine ("SparkleShare is a Free and Open Source software program that helps people ");
-            writer.WriteLine ("collaborate and share files. If you like what we do, please consider a small ");
-            writer.WriteLine ("donation to support the project: http://sparkleshare.org/support-us/");
-            writer.WriteLine ("");
-            writer.WriteLine ("Have fun! :)");
-            writer.WriteLine ("");
-
-            writer.Close ();
-
-            SyncUp ();
-        }
-
-
-        // Creates a SHA-1 hash of input
-        private string SHA1 (string s)
-        {
-            SHA1 sha1 = new SHA1CryptoServiceProvider ();
-            Byte[] bytes = ASCIIEncoding.Default.GetBytes (s);
-            Byte[] encoded_bytes = sha1.ComputeHash (bytes);
-            return BitConverter.ToString (encoded_bytes).ToLower ().Replace ("-", "");
-        }
-
-
         // Recursively gets a folder's size in bytes
         private double CalculateSize (DirectoryInfo parent)
         {
@@ -554,7 +510,7 @@ namespace SparkleLib {
                 return 0;
 
             try {
-                foreach (FileInfo file in parent.GetFiles()) {
+                foreach (FileInfo file in parent.GetFiles ()) {
                     if (!file.Exists)
                         return 0;
 
@@ -570,5 +526,12 @@ namespace SparkleLib {
 
             return size;
         }
+
+
+        public void Dispose ()
+        {
+            this.remote_timer.Dispose ();
+            this.listener.Dispose ();
+        }
     }
 }
